#include <assert.h>           // assert()
#include <stdlib.h>           // malloc()
#include <stdio.h>            // fprintf()
#include <strings.h>          // bzero()
#include <sys/select.h>       // select()
#include <errno.h>            // errno

#include "mDNS/mDNSCore/mDNSClientAPI.h"  // defines the interface to the client layer above
#include "mDNS/mDNSPosix/mDNSPosix.h"     // defines the specific types needed on this platform

#include "mdns.h"

const char mdns_kDefaultPIDFile[]       = "/var/run/iCast.pid";
const char mdns_kDefaultServiceType[]   = "_daap._tcp.";
const char mdns_kDefaultServiceDomain[] = "local.";
const char mdns_kDefaultServiceRoot[]   = "./files";
const char mdns_kDefaultServicePass[]   = "corvette";
const char mdns_kDefaultServiceName[]   = "iCast";
const int  mdns_kDefaultServicePort     = 5090;

#if COMPILER_LIKES_PRAGMA_MARK
#pragma mark ***** Private Methods [Registration]
#endif

static void
mdns_RegistrationCallback (m, thisreg, status)
  mDNS * const m;
  ServiceRecordSet * const thisreg;
  mStatus status;
{
  (void)m;
  (void)thisreg;
  (void)status;
  
  fprintf(stderr, "mdns_RegistrationCallback().. I'm here.\n");
}

// static mStatus
// mdns_RegisterService (mdns, serviceName, serviceType, serviceDomain, text, textLen, portNumber)
//   mdns_t * mdns;
//   const char * serviceName;
//   const char * serviceType;
//   const char * serviceDomain;
//   const mDNSu8 text[];
//   mDNSu16      textLen;
//   long         portNumber;
// {

static mStatus
mdns_RegisterService (mdns)
  mdns_t * mdns;
{
  mStatus             status;
  mdns_PosixService * thisServ;
  mDNSOpaque16        port;
  domainlabel         name;
  domainname          type;
  domainname          domain;
  
  if (NULL == (thisServ = (mdns_PosixService *)malloc(sizeof(*thisServ))))
    return mStatus_NoMemoryErr;

//MakeDomainLabelFromLiteralString(&name,  serviceName);
//MakeDomainNameFromDNSNameString(&type,   serviceType);
//MakeDomainNameFromDNSNameString(&domain, serviceDomain);
  
  MakeDomainLabelFromLiteralString(&name,  (char *)&mdns->srvc_name);
  MakeDomainNameFromDNSNameString(&type,   (char *)&mdns->srvc_type);
  MakeDomainNameFromDNSNameString(&domain, (char *)&mdns->srvc_host);

//port.b[0] = (portNumber >> 8) & 0x0FF;
//port.b[1] = (portNumber >> 0) & 0x0FF;

  port.b[0] = (mdns->srvc_port >> 8) & 0x0FF;
  port.b[1] = (mdns->srvc_port >> 0) & 0x0FF;
  
  status = mDNS_RegisterService (
//  &mdns_mDNSStorage,                  // where the core stores its stuff
    &mdns->mDNSStorage,
    &thisServ->coreServ,                // something important, i'm sure
    &name, &type, &domain,              // name, type, domain
    NULL, port,                         // host and port
    (char *)&mdns->srvc_text,           // text data
    mdns->text_leng,                    // text length
    NULL, 0,                            // subtypes
    mDNSInterface_Any,                  // interface id
    mdns_RegistrationCallback,          // callback function
    thisServ);                          // context
  
  if (status != mStatus_NoError) {
    if (thisServ != NULL) free(thisServ);
    return status;
  }
  
//thisServ->serviceID = mdns_gServiceID;
//mdns_gServiceID += 1;
  thisServ->serviceID = mdns->ServiceID;
  mdns->ServiceID += 1;
  
//thisServ->next = mdns_gServiceList;
//mdns_gServiceList = thisServ;
  thisServ->next = mdns->ServiceList;
  mdns->ServiceList = thisServ;
  
  fprintf(stderr, "%s: Registered service %d, name '%s', type '%s', port %ld\n",
          (char *)&mdns->srvc_name,
           mdns->ServiceID,
          (char *)&mdns->srvc_name,
          (char *)&mdns->srvc_type,
           mdns->srvc_port);
  
//        mdns_gProgramName,
//        thisServ->serviceID,
//        serviceName,
//        serviceType,
//        portNumber);
  
  return mStatus_NoError;
}

#if COMPILER_LIKES_PRAGMA_MARK
#pragma mark ***** Public Methods
#endif

mdns_t *
mdns_Create ()
{
  mdns_t * mdns;
  
  mdns = (mdns_t *)malloc(sizeof(mdns_t));
  assert(mdns != NULL);
  
  return mdns;
}

void
mdns_Init (mdns)
  mdns_t * mdns;
{
  assert(mdns != NULL);
  
  mdns->ServiceList = NULL;
  mdns->ServiceID   = 0;
  mdns->text_leng   = 0;

  bzero(&mdns->pid_file,  32);
  bzero(&mdns->srvc_type, 32);
  bzero(&mdns->srvc_host, 32);
  bzero(&mdns->srvc_root, 32);
  bzero(&mdns->srvc_pass, 32);
  bzero(&mdns->srvc_name, 32);
  
  memcpy(&mdns->pid_file,  &mdns_kDefaultPIDFile,       strlen(mdns_kDefaultPIDFile));
  memcpy(&mdns->srvc_type, &mdns_kDefaultServiceType,   strlen(mdns_kDefaultServiceType));
  memcpy(&mdns->srvc_host, &mdns_kDefaultServiceDomain, strlen(mdns_kDefaultServiceDomain));
  memcpy(&mdns->srvc_root, &mdns_kDefaultServiceRoot,   strlen(mdns_kDefaultServiceRoot));
  memcpy(&mdns->srvc_pass, &mdns_kDefaultServicePass,   strlen(mdns_kDefaultServicePass));
  memcpy(&mdns->srvc_name, &mdns_kDefaultServiceName,   strlen(mdns_kDefaultServiceName));

  mdns->srvc_port = mdns_kDefaultServicePort;
  
  fprintf(stderr, "mdns_Init.. pid_file  = %s\n", (char *)&mdns->pid_file );
  fprintf(stderr, "mdns_Init.. srvc_type = %s\n", (char *)&mdns->srvc_type);
  fprintf(stderr, "mdns_Init.. srvc_host = %s\n", (char *)&mdns->srvc_host);
  fprintf(stderr, "mdns_Init.. srvc_root = %s\n", (char *)&mdns->srvc_root);
  fprintf(stderr, "mdns_Init.. srvc_pass = %s\n", (char *)&mdns->srvc_pass);
  fprintf(stderr, "mdns_Init.. srvc_name = %s\n", (char *)&mdns->srvc_name);
  fprintf(stderr, "mdns_Init.. srvc_port = %ld\n",         mdns->srvc_port);
}

void
mdns_Start (mdns)
  mdns_t * mdns;
{
  mStatus status;
  
  assert(mdns != NULL);

  if (mdns->started != 0) {
    fprintf(stderr, "mdns_Start.. already running\n");
    return;
  }
  
  status = mDNS_Init (
//  &mdns_mDNSStorage,
    &mdns->mDNSStorage,
//  &mdns_PlatformStorage,
    &mdns->PlatformStorage,
    mDNS_Init_NoCache,
    mDNS_Init_ZeroCacheSize,
    mDNS_Init_AdvertiseLocalAddresses,
    mDNS_Init_NoInitCallback,
    mDNS_Init_NoInitCallbackContext);

  assert(status == mStatus_NoError);
  
  mdns_RegisterService (mdns);
//  mdns_kDefaultServiceName,
//  &mdns->srvc_name,
//  mdns_kDefaultServiceType,
//  &mdns->srvc_type,
//  mdns_kDefaultServiceDomain,
//  &mdns->srvc_host,
//  mdns_gServiceText,
//  &mdns->srvc_text,
//  mdns_gServiceTextLen,
//  mdns->text_leng,
//  mdns_kDefaultPortNumber);
//  mdns->srvc_port);
    
  status = pthread_create(&mdns->thread, NULL, mdns_Loop, mdns);
  assert(status == 0);
}

void
mdns_Stop (mdns)
  mdns_t * mdns;
{
  assert(mdns != NULL);
  
  if (mdns->started != 1) {
    fprintf(stderr, "mdns_Stop.. not running\n");
    return;
  }
  
}

void *
mdns_Loop (arg)
  void * arg;
{
  int nfds = 0;
  fd_set readfds;
  struct timeval timeout;
  int result;
  mdns_t * mdns = (mdns_t *)arg;
  
  assert(mdns != NULL);
  
  while (mdns->hault == 0)
  {
    FD_ZERO(&readfds);
    
    timeout.tv_sec  = 0x00000001;
    timeout.tv_usec = 0x00;
    
//  mDNSPosixGetFDSet(&mdns_mDNSStorage, &nfds, &readfds, &timeout);
    mDNSPosixGetFDSet(&mdns->mDNSStorage, &nfds, &readfds, &timeout);
    
    if (0 <= (result = select(nfds, &readfds, NULL, NULL, &timeout)))
//    mDNSPosixProcessFDSet(&mdns_mDNSStorage, &readfds);
      mDNSPosixProcessFDSet(&mdns->mDNSStorage, &readfds);
    else
      fprintf(stderr, "Error.. select() returned %d errno %d\n", result, errno);
  }
  
  fprintf(stderr, "mdns_Loop.. Exiting.\n");
  
//mDNS_DeregisterService(&mdns_mDNSStorage, &mdns_gServiceList->coreServ);
//mDNS_Close(&mdns_mDNSStorage);
  mDNS_DeregisterService(&mdns->mDNSStorage, &mdns->ServiceList->coreServ);
  mDNS_Close(&mdns->mDNSStorage);
  
  return NULL;
}

void
mdns_Destroy (mdns)
  mdns_t * mdns;
{
  assert(mdns != NULL);
  
  mDNS_DeregisterService(&mdns->mDNSStorage, &mdns->ServiceList->coreServ);
  mDNS_Close(&mdns->mDNSStorage);
}
